"""Test harness for StandaloneZODB

The test harness was written for Python 2.1 and requires some hand-holding to
use with older Python versions.  There's probably not much hope of passing the
test suite with anything earlier than Python 2.0.  If you are trying to use
Python 2.0 you will need a recent version of unittest, which can be downloaded
from pyunit.sourceforge.net.
     
If the ZEO tests are to be run in older Python versions, you will need a built
version of Zope 2.3, which provides up-to-date versions of asyncore and
cPickle.

There are some complications, however, because the StandaloneZODB includes
code from Zope 2.4, which requires Python 2.1.  If the Zope top-level
directory and lib/python directories are added to the path, the BTrees tests
will fail, because they pick up an old version of the Interface module.  But
the ZEO tests won't run *without* Zope on the path.  So it's one or the other.
"""

import gc
import os
import re
import string
import sys
import unittest
from distutils.util import get_platform

# setup list of directories to put on the path

def setup_path():
    PLAT_SPEC = "%s-%s" % (get_platform(), sys.version[0:3])
    DIRS = ["lib",
            "lib.%s" % PLAT_SPEC,
            "../ExtensionClass/build/lib.%s" % PLAT_SPEC,
            ]
    for d in DIRS:
        sys.path.insert(0, d)

# find test files
class TestFileFinder:
    def __init__(self):
        self.files = []

    def visit(self, rx, dir, files):
        if dir[-5:] != "tests":
            return
        # ignore tests that aren't in packages
        if not "__init__.py" in files:
            print "not a package", dir
            return
        for file in files:
            if file[:4] == "test" and file[-3:] == ".py":
                path = os.path.join(dir, file)
                if rx is not None:
                    if rx.search(path):
                        self.files.append(path)
                else:
                    self.files.append(path)

def find_tests(filter):
    if filter is not None:
        rx = re.compile(filter)
    else:
        rx = None
    finder = TestFileFinder()
    os.path.walk("build", finder.visit, rx)
    return finder.files

def package_import(modname):
    mod = __import__(modname)
    for part in string.split(modname, ".")[1:]:
        mod = getattr(mod, part)
    return mod

def module_from_path(path):
    """Return the Python package name indiciated by the filesystem path.

    The path starts with build/lib or build /lib.mumble..."""

    assert path[-3:] == '.py'
    path = path[:-3]
    dirs = []
    while path:
        path, end = os.path.split(path)
        dirs.insert(0, end)
    assert dirs[0] == "build"
    assert dirs[1][:3] == "lib"
    return string.join(dirs[2:], '.')

if sys.platform == "linux2":
    _pid = os.getpid()
    
    def check_mem():
        vm = {}
        order = []
        f = open("/proc/%d/status" % _pid)
        for line in f.readlines():
            if line[:2] == 'Vm':
                field, val = string.split(line, ':', 2)
                val = string.strip(val)
                vm[field] = val
                order.append((field, max(len(field), len(val))))
        f.close()
        for field, size in order:
            fmt = "%%%ds " % size
            print fmt % field,
        print
        for field, size in order:
            fmt = "%%%ds " % size
            print fmt % vm[field],
        print
else:
    # not sure what to do on any other platform
    def check_mem():
        pass

def get_suite(file):
    assert file[:5] == "build"
    assert file[-3:] == '.py'
    modname = module_from_path(file)
    try:
        mod = package_import(modname)
    except Exception, e:
        print '%s skipped: %s' % (modname, e)
        return None
    try:
        return mod.test_suite()
    except AttributeError:
        return None

def main(filter=None):
    setup_path()
    files = find_tests(filter)

    os.chdir("build") 

    if LOOP:
        suites = []
        for file in files:
            suite = get_suite(file)
            if suite is None:
                continue
            suites.append(suite)
        if suites:
            runner = unittest.TextTestRunner(verbosity=VERBOSE)
            while 1:
                for suite in suites:
                    runner.run(suite)
                gc.collect()
                check_mem()
    else:
        runner = unittest.TextTestRunner(verbosity=VERBOSE)
        if MERGE_SUITES:
            suite = unittest.TestSuite()
            for file in files:
                s = get_suite(file)
                if s is not None:
                    suite.addTest(s)
            runner.run(suite)
        else:
            for file in files:
                suite = get_suite(file)
                if suite is None:
                    continue
                runner.run(suite)

if __name__ == "__main__":
    import getopt

    filter = None
    VERBOSE = 0
    LOOP = 0
    MERGE_SUITES = 1

    opts, args = getopt.getopt(sys.argv[1:], 'vLi')
    for k, v in opts:
        if k == '-v':
            VERBOSE = VERBOSE + 1
        elif k == '-L':
            LOOP = 1
        elif k == '-i':
            MERGE_SUITES = 0
    if args:
        filter = args[0]
    try:
        main(filter)
    except ImportError, err:
        print err
        print sys.path
        raise
